==> ejc-zdsstore-impl-3.0.0-20231211.094140-6-sources.jar: com/ejianc/business/zdsstore/service/handler/TurnOutStoreHandler.java <==
package com.ejianc.business.zdsstore.service.handler;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ejianc.business.zdsstore.bean.FlowEntity;
import com.ejianc.business.zdsstore.bean.InOutEntity;
import com.ejianc.business.zdsstore.bean.SurplusEntity;
import com.ejianc.business.zdsstore.consts.StoreCommonConsts;
import com.ejianc.business.zdsstore.service.IFlowService;
import com.ejianc.business.zdsstore.service.IInOutService;
import com.ejianc.business.zdsstore.service.IStoreService;
import com.ejianc.business.zdsstore.service.ISurplusService;
import com.ejianc.business.zdsstore.util.StoreLockUtil;
import com.ejianc.business.zdsstore.vo.*;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.util.ComputeUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author songlx
 * @version 1.0
 * @description: 周转材调拨出库/处置
 * @date 2022/1/25
 */
@Service
public class TurnOutStoreHandler implements IStoreManageHandler {

    @Autowired
    IFlowService flowService;

    @Autowired
    IInOutService inOutService;

    @Autowired
    ISurplusService surplusService;

    @Autowired
    IStoreService storeService;

    /**
     * @param storeManageVO
     * @description: 出库具体实现逻辑
     * 1 生效的话根据出入库关系将锁定量变成出库量
     * @return: com.ejianc.framework.core.response.CommonResponse<com.ejianc.business.store.vo.StoreManageVO>
     * @author songlx
     * @date: 2022/1/25
     */
    @Override
    public CommonResponse<StoreManageVO> handle(StoreManageVO storeManageVO) {
        Long sourceId = storeManageVO.getSourceId();
        Long storeId = storeManageVO.getStoreId();
        Boolean lock = StoreLockUtil.getLock(storeId);
        //获取出库生效标识,如果是生效,则无需校验库存,只改下占用状态即可
        Boolean outEffectiveON = storeManageVO.getOutEffectiveON();

        List<FlowEntity> inFlowEntityList = new ArrayList<>();
        if (outEffectiveON) {
            QueryWrapper<InOutEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("out_bill_id", sourceId);
            //生效状态 未生效
            queryWrapper.eq("effective_state", StoreCommonConsts.NO);
            List<InOutEntity> inOutEntityList = inOutService.list(queryWrapper);
            List<Long> inFlowList = inOutEntityList.stream().map(InOutEntity::getInFlowId).collect(Collectors.toList());
            List<Long> materialIds = inOutEntityList.stream().map(InOutEntity::getMaterialId).collect(Collectors.toList());
            List<FlowEntity> flowEntities = (List<FlowEntity>) flowService.listByIds(inFlowList);
            Map<Long, List<FlowEntity>> inFlowMap = flowEntities.stream().collect(Collectors.groupingBy(FlowEntity::getId));

            Map<Long, SurplusEntity> surplusUpdateMap = new HashMap<>();
            QueryWrapper<SurplusEntity> surplusEntityQueryWrapper = new QueryWrapper<>();
            surplusEntityQueryWrapper.eq("store_id", storeId);
            surplusEntityQueryWrapper.in("material_id", materialIds);
            List<SurplusEntity> surplusEntities = surplusService.list(surplusEntityQueryWrapper);
            //原来的库存
            surplusEntities.forEach(t -> surplusUpdateMap.put(t.getMaterialId(), t));
            List<SurplusEntity> updateSurplusList = new ArrayList<>();
            inOutEntityList.forEach(t -> {
                //原锁定数量\金额
                BigDecimal effectOutNum = t.getOutLockNum();
                BigDecimal outNetMny = t.getOutNetMny();
                BigDecimal outNetTaxMny = t.getOutNetTaxMny();
                t.setEffectiveDate(new Date());
                t.setEffectiveState(StoreCommonConsts.YES);
                //生效后占用数量归集到出库数量,占用数量归于0
                t.setOutNum(effectOutNum);
                t.setOutLockNum(BigDecimal.ZERO);
                t.setOutUseFlag(StoreCommonConsts.UseOutFlag.USE_FINISH);
                //更新库存锁定,生效后 锁定数量减少
                SurplusEntity surplusVO = surplusUpdateMap.get(t.getMaterialId());
                BigDecimal outLockNum = ComputeUtil.safeSub(surplusVO.getOutLockNum(), effectOutNum);
                surplusVO.setOutLockNum(outLockNum);
                BigDecimal outLockMny = ComputeUtil.safeSub(surplusVO.getOutLockMny(), outNetMny);
                surplusVO.setOutLockMny(outLockMny);
                BigDecimal outLockTaxMny = ComputeUtil.safeSub(surplusVO.getOutLockTaxMny(), outNetTaxMny);
                surplusVO.setOutLockMny(outLockTaxMny);

                updateSurplusList.add(surplusVO);
                //入库的锁定变成出库
                List<FlowEntity> flowVOS = inFlowMap.get(t.getInFlowId());
                FlowEntity flowEntity = flowVOS.get(0);
                BigDecimal _outLockNum = ComputeUtil.safeSub(flowEntity.getOutLockNum(), effectOutNum);
                flowEntity.setOutLockNum(_outLockNum);
                BigDecimal _outNum = ComputeUtil.safeAdd(flowEntity.getOutNum(), effectOutNum);
                flowEntity.setOutNum(_outNum);
                //周转材出库净值
                BigDecimal sumOutNetMny = ComputeUtil.safeAdd(flowEntity.getOutNetMny(), outNetMny);
                flowEntity.setOutNetMny(sumOutNetMny);
                BigDecimal sumOutNetTaxMny = ComputeUtil.safeAdd(flowEntity.getOutNetTaxMny(), outNetTaxMny);
                flowEntity.setOutNetTaxMny(sumOutNetTaxMny);
                inFlowEntityList.add(flowEntity);
            });

            //出库单生效.则出库单生效状态由占用态变成生效态
            QueryWrapper<FlowEntity> outFlowQueryWrapper = new QueryWrapper<>();
            outFlowQueryWrapper.eq("source_id", sourceId);
            List<FlowEntity> outFlowList = flowService.list(outFlowQueryWrapper);
            outFlowList.forEach(t -> {
                t.setEffectiveDate(new Date());
                t.setEffectiveState(StoreCommonConsts.YES);
                inFlowEntityList.add(t);
            });

            flowService.saveOrUpdateBatch(inFlowEntityList);
            inOutService.saveOrUpdateBatch(inOutEntityList);
            surplusService.saveOrUpdateBatch(updateSurplusList);
            return CommonResponse.success(storeManageVO);
        }
        List<FlowVO> flowVOList = storeManageVO.getFlowVOList();
        CommonResponse<SurplusUpdateVO> res;
        try {
            //1 校验库存量 要依据正数的变化量来和库存余量校验
            res = surplusService.validateStoreSurplusByOutLock(storeManageVO);
            if (res.isSuccess()) {
                //先进先出占用计算
                UseCalculateVO useCalculateVO = surplusService.useTurnCalculate(storeManageVO);
                if (useCalculateVO == null) {
                    return CommonResponse.success("材料出库明细数量没有变化,仓库无需处理!");
                }
                // 查询是否已有对应的出库流水, 有则覆盖,无则插入
                List<FlowVO> updateFlowVOList = useCalculateVO.getUpdateFlowVOList();
                QueryWrapper<FlowEntity> outFlowWrapper = new QueryWrapper<>();
                outFlowWrapper.eq("source_id", sourceId);
                List<FlowEntity> outstoreList = flowService.list(outFlowWrapper);
                if (CollectionUtils.isNotEmpty(outstoreList)) {
                    Map<Long, List<FlowEntity>> outstoreMap = outstoreList.stream().collect(Collectors.groupingBy(FlowEntity::getSourceDetailId));
                    flowVOList.forEach(
                            t -> {
                                Long sourceDetailId = t.getSourceDetailId();
                                List<FlowEntity> flowEntityList = outstoreMap.get(sourceDetailId);
                                if (CollectionUtils.isNotEmpty(flowEntityList)) {
                                    FlowEntity flowEntity = flowEntityList.get(0);
                                    t.setId(flowEntity.getId());
                                    t.setVersion(flowEntity.getVersion());
                                }
                            }
                    );
                }

                // 出库流水合并进来
                updateFlowVOList.addAll(flowVOList);
                if (CollectionUtils.isNotEmpty(updateFlowVOList)) {
                    List<FlowEntity> flowEntities = BeanMapper.mapList(updateFlowVOList, FlowEntity.class);
                    ArrayList<FlowEntity> updateFlowList = new ArrayList<>();
                    ArrayList<FlowEntity> delFlowList = new ArrayList<>();
                    flowEntities.forEach(n -> {
                        if ("del".equals(n.getRowState())) {
                            delFlowList.add(n);
                        } else {
                            updateFlowList.add(n);
                        }
                    });
                    if (CollectionUtils.isNotEmpty(delFlowList)) {
                        flowService.removeByIds(delFlowList.stream().map(FlowEntity::getId).collect(Collectors.toList()));
                    }
                    if (CollectionUtils.isNotEmpty(updateFlowList)) {
                        flowService.saveOrUpdateBatch(updateFlowList);
                    }
                }
                List<InOutVO> updateInOutVOList = useCalculateVO.getUpdateInOutVOList();
                if (CollectionUtils.isNotEmpty(updateInOutVOList)) {
                    List<InOutEntity> inOutEntities = BeanMapper.mapList(updateInOutVOList, InOutEntity.class);
                    //根据dr分组,因为有的需要删除,有的是修改
                    Map<Integer, List<InOutEntity>> list = inOutEntities.stream().collect(Collectors.groupingBy(InOutEntity::getDr));
                    List<InOutEntity> delList = list.get(1);
                    List<InOutEntity> saveOrUpdList = list.get(0);
                    if (CollectionUtils.isNotEmpty(delList)) {
                        inOutService.removeByIds(delList.stream().map(InOutEntity::getId).collect(Collectors.toList()));
                    }
                    if (CollectionUtils.isNotEmpty(saveOrUpdList)) {
                        inOutService.saveOrUpdateBatch(saveOrUpdList);
                    }

                }
                List<SurplusVO> updateSurplusVOList = useCalculateVO.getUpdateSurplusVOList();
                if (CollectionUtils.isNotEmpty(updateSurplusVOList)) {
                    List<SurplusEntity> surplusEntities = BeanMapper.mapList(updateSurplusVOList, SurplusEntity.class);
                    surplusService.saveOrUpdateBatch(surplusEntities);
                }
                return CommonResponse.success(storeManageVO);
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("操作异常，请刷新重试！");
        } finally {
            if (lock) {
                StoreLockUtil.releaseLock(storeId);
            }
        }
        return CommonResponse.error(res.getMsg());
    }

    /**
     * @param storeManageVO
     * @description: 仓库出库具体实现逻辑逆向实现, 主要调用:出库单删除时
     * @return: com.ejianc.framework.core.response.CommonResponse<com.ejianc.business.store.vo.StoreManageVO>
     * @author songlx
     * @date: 2022/1/25
     */
    @Override
    public CommonResponse<StoreManageVO> handleRollback(StoreManageVO storeManageVO) {
        Long storeId = storeManageVO.getStoreId();
        Boolean lock = StoreLockUtil.getLock(storeId);
        try {
            UseCalculateVO useCalculateVO = surplusService.turnOutRollBackByInOut(storeManageVO, storeManageVO.getOutEffectiveON());
            //3 出库后更新入库\出入关系\库存
            List<FlowVO> updateFlowVOList = useCalculateVO.getUpdateFlowVOList();
            if (CollectionUtils.isNotEmpty(updateFlowVOList)) {
                List<FlowEntity> flowEntities = BeanMapper.mapList(updateFlowVOList, FlowEntity.class);
                flowService.saveOrUpdateBatch(flowEntities);
                // 出库单的处理
                List<Long> sourceIdsForRollBack = storeManageVO.getSourceIdsForRollBack();
                QueryWrapper<FlowEntity> outFlowQueryWrapper = new QueryWrapper<>();
                outFlowQueryWrapper.in("source_id", sourceIdsForRollBack);
                List<FlowEntity> outFlowList = flowService.list(outFlowQueryWrapper);

                if (storeManageVO.getOutEffectiveON()) {
                    outFlowList.forEach(t -> {
                        t.setEffectiveDate(new Date());
                        t.setEffectiveState(StoreCommonConsts.NO);
                    });
                    //出库单生效弃审回滚.则出库单生效状态变成占用态
                    flowService.saveOrUpdateBatch(outFlowList);
                } else {
                    //出库单自由态删除回滚.则删除出库单
                    List<Long> delListIds = outFlowList.stream().map(FlowEntity::getId).collect(Collectors.toList());
                    flowService.removeByIds(delListIds);
                }
            }
            List<InOutVO> updateInOutVOList = useCalculateVO.getUpdateInOutVOList();
            if (CollectionUtils.isNotEmpty(updateInOutVOList)) {
                List<InOutEntity> inOutEntities = BeanMapper.mapList(updateInOutVOList, InOutEntity.class);
                if (storeManageVO.getOutEffectiveON()) {
                    inOutEntities.forEach(t -> {
                        t.setEffectiveDate(new Date());
                        t.setEffectiveState(StoreCommonConsts.NO);
                    });
                    inOutService.saveOrUpdateBatch(inOutEntities);
                } else {
                    List<Long> delListIds = inOutEntities.stream().map(InOutEntity::getId).collect(Collectors.toList());
                    inOutService.removeByIds(delListIds);
                }
            }
            List<SurplusVO> updateSurplusVOList = useCalculateVO.getUpdateSurplusVOList();
            if (CollectionUtils.isNotEmpty(updateSurplusVOList)) {
                List<SurplusEntity> surplusEntities = BeanMapper.mapList(updateSurplusVOList, SurplusEntity.class);
                surplusService.saveOrUpdateBatch(surplusEntities);
            }

            return CommonResponse.success(storeManageVO);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("操作异常，请刷新重试！");
        } finally {
            if (lock) {
                StoreLockUtil.releaseLock(storeId);
            }
        }
    }

}

==> ejc-zdsstore-impl-3.0.0-20231221.065828-13-sources.jar: com/ejianc/business/zdsstore/service/handler/TurnOutStoreHandler.java <==
package com.ejianc.business.zdsstore.service.handler;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ejianc.business.zdsstore.bean.FlowEntity;
import com.ejianc.business.zdsstore.bean.InOutEntity;
import com.ejianc.business.zdsstore.bean.SurplusEntity;
import com.ejianc.business.zdsstore.consts.StoreCommonConsts;
import com.ejianc.business.zdsstore.service.IFlowService;
import com.ejianc.business.zdsstore.service.IInOutService;
import com.ejianc.business.zdsstore.service.IStoreService;
import com.ejianc.business.zdsstore.service.ISurplusService;
import com.ejianc.business.zdsstore.util.StoreLockUtil;
import com.ejianc.business.zdsstore.vo.*;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.core.util.ComputeUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author songlx
 * @version 1.0
 * @description: 周转材调拨出库/处置
 * @date 2022/1/25
 */
@Service
public class TurnOutStoreHandler implements IStoreManageHandler {

    @Autowired
    IFlowService flowService;

    @Autowired
    IInOutService inOutService;

    @Autowired
    ISurplusService surplusService;

    @Autowired
    IStoreService storeService;

    /**
     * @param storeManageVO
     * @description: 出库具体实现逻辑
     * 1 生效的话根据出入库关系将锁定量变成出库量
     * @return: com.ejianc.framework.core.response.CommonResponse<com.ejianc.business.store.vo.StoreManageVO>
     * @author songlx
     * @date: 2022/1/25
     */
    @Override
    public CommonResponse<StoreManageVO> handle(StoreManageVO storeManageVO) {
        Long sourceId = storeManageVO.getSourceId();
        Long storeId = storeManageVO.getStoreId();
        Boolean lock = StoreLockUtil.getLock(storeId);
        //获取出库生效标识,如果是生效,则无需校验库存,只改下占用状态即可
        Boolean outEffectiveON = storeManageVO.getOutEffectiveON();

        List<FlowEntity> inFlowEntityList = new ArrayList<>();
        if (outEffectiveON) {
            QueryWrapper<InOutEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("out_bill_id", sourceId);
            //生效状态 未生效
            queryWrapper.eq("effective_state", StoreCommonConsts.NO);
            List<InOutEntity> inOutEntityList = inOutService.list(queryWrapper);
            List<Long> inFlowList = inOutEntityList.stream().map(InOutEntity::getInFlowId).collect(Collectors.toList());
            List<Long> materialIds = inOutEntityList.stream().map(InOutEntity::getMaterialId).collect(Collectors.toList());
            List<Long> brandIds = inOutEntityList.stream().map(InOutEntity::getBrandId).collect(Collectors.toList());
            List<FlowEntity> flowEntities = (List<FlowEntity>) flowService.listByIds(inFlowList);
            Map<Long, List<FlowEntity>> inFlowMap = flowEntities.stream().collect(Collectors.groupingBy(FlowEntity::getId));

            Map<String, SurplusEntity> surplusUpdateMap = new HashMap<>();
            QueryWrapper<SurplusEntity> surplusEntityQueryWrapper = new QueryWrapper<>();
            surplusEntityQueryWrapper.eq("store_id", storeId);
            surplusEntityQueryWrapper.in("material_id", materialIds);
            surplusEntityQueryWrapper.in("brand_id", brandIds);
            List<SurplusEntity> surplusEntities = surplusService.list(surplusEntityQueryWrapper);
            //原来的库存
            surplusEntities.forEach(t -> surplusUpdateMap.put(t.getMaterialId() + "@" + t.getBrandId(), t));
            List<SurplusEntity> updateSurplusList = new ArrayList<>();
            inOutEntityList.forEach(t -> {
                //原锁定数量\金额
                BigDecimal effectOutNum = t.getOutLockNum();
                BigDecimal outNetMny = t.getOutNetMny();
                BigDecimal outNetTaxMny = t.getOutNetTaxMny();
                t.setEffectiveDate(new Date());
                t.setEffectiveState(StoreCommonConsts.YES);
                //生效后占用数量归集到出库数量,占用数量归于0
                t.setOutNum(effectOutNum);
                t.setOutLockNum(BigDecimal.ZERO);
                t.setOutUseFlag(StoreCommonConsts.UseOutFlag.USE_FINISH);
                //更新库存锁定,生效后 锁定数量减少
                SurplusEntity surplusVO = surplusUpdateMap.get(t.getMaterialId() + "@" + t.getBrandId());
                BigDecimal outLockNum = ComputeUtil.safeSub(surplusVO.getOutLockNum(), effectOutNum);
                surplusVO.setOutLockNum(outLockNum);
                BigDecimal outLockMny = ComputeUtil.safeSub(surplusVO.getOutLockMny(), outNetMny);
                surplusVO.setOutLockMny(outLockMny);
                BigDecimal outLockTaxMny = ComputeUtil.safeSub(surplusVO.getOutLockTaxMny(), outNetTaxMny);
                surplusVO.setOutLockMny(outLockTaxMny);

                updateSurplusList.add(surplusVO);
                //入库的锁定变成出库
                List<FlowEntity> flowVOS = inFlowMap.get(t.getInFlowId());
                FlowEntity flowEntity = flowVOS.get(0);
                BigDecimal _outLockNum = ComputeUtil.safeSub(flowEntity.getOutLockNum(), effectOutNum);
                flowEntity.setOutLockNum(_outLockNum);
                BigDecimal _outNum = ComputeUtil.safeAdd(flowEntity.getOutNum(), effectOutNum);
                flowEntity.setOutNum(_outNum);
                //周转材出库净值
                BigDecimal sumOutNetMny = ComputeUtil.safeAdd(flowEntity.getOutNetMny(), outNetMny);
                flowEntity.setOutNetMny(sumOutNetMny);
                BigDecimal sumOutNetTaxMny = ComputeUtil.safeAdd(flowEntity.getOutNetTaxMny(), outNetTaxMny);
                flowEntity.setOutNetTaxMny(sumOutNetTaxMny);
                inFlowEntityList.add(flowEntity);
            });

            //出库单生效.则出库单生效状态由占用态变成生效态
            QueryWrapper<FlowEntity> outFlowQueryWrapper = new QueryWrapper<>();
            outFlowQueryWrapper.eq("source_id", sourceId);
            List<FlowEntity> outFlowList = flowService.list(outFlowQueryWrapper);
            outFlowList.forEach(t -> {
                t.setEffectiveDate(new Date());
                t.setEffectiveState(StoreCommonConsts.YES);
                inFlowEntityList.add(t);
            });

            flowService.saveOrUpdateBatch(inFlowEntityList);
            inOutService.saveOrUpdateBatch(inOutEntityList);
            surplusService.saveOrUpdateBatch(updateSurplusList);
            return CommonResponse.success(storeManageVO);
        }
        List<FlowVO> flowVOList = storeManageVO.getFlowVOList();
        CommonResponse<SurplusUpdateVO> res;
        try {
            //1 校验库存量 要依据正数的变化量来和库存余量校验
            res = surplusService.validateStoreSurplusByOutLock(storeManageVO);
            if (res.isSuccess()) {
                //先进先出占用计算
                UseCalculateVO useCalculateVO = surplusService.useTurnCalculate(storeManageVO);
                if (useCalculateVO == null) {
                    return CommonResponse.success("材料出库明细数量没有变化,仓库无需处理!");
                }
                // 查询是否已有对应的出库流水, 有则覆盖,无则插入
                List<FlowVO> updateFlowVOList = useCalculateVO.getUpdateFlowVOList();
                QueryWrapper<FlowEntity> outFlowWrapper = new QueryWrapper<>();
                outFlowWrapper.eq("source_id", sourceId);
                List<FlowEntity> outstoreList = flowService.list(outFlowWrapper);
                if (CollectionUtils.isNotEmpty(outstoreList)) {
                    Map<Long, List<FlowEntity>> outstoreMap = outstoreList.stream().collect(Collectors.groupingBy(FlowEntity::getSourceDetailId));
                    flowVOList.forEach(
                            t -> {
                                Long sourceDetailId = t.getSourceDetailId();
                                List<FlowEntity> flowEntityList = outstoreMap.get(sourceDetailId);
                                if (CollectionUtils.isNotEmpty(flowEntityList)) {
                                    FlowEntity flowEntity = flowEntityList.get(0);
                                    t.setId(flowEntity.getId());
                                    t.setVersion(flowEntity.getVersion());
                                }
                            }
                    );
                }

                // 出库流水合并进来
                updateFlowVOList.addAll(flowVOList);
                if (CollectionUtils.isNotEmpty(updateFlowVOList)) {
                    List<FlowEntity> flowEntities = BeanMapper.mapList(updateFlowVOList, FlowEntity.class);
                    ArrayList<FlowEntity> updateFlowList = new ArrayList<>();
                    ArrayList<FlowEntity> delFlowList = new ArrayList<>();
                    flowEntities.forEach(n -> {
                        if ("del".equals(n.getRowState())) {
                            delFlowList.add(n);
                        } else {
                            updateFlowList.add(n);
                        }
                    });
                    if (CollectionUtils.isNotEmpty(delFlowList)) {
                        flowService.removeByIds(delFlowList.stream().map(FlowEntity::getId).collect(Collectors.toList()));
                    }
                    if (CollectionUtils.isNotEmpty(updateFlowList)) {
                        flowService.saveOrUpdateBatch(updateFlowList);
                    }
                }
                List<InOutVO> updateInOutVOList = useCalculateVO.getUpdateInOutVOList();
                if (CollectionUtils.isNotEmpty(updateInOutVOList)) {
                    List<InOutEntity> inOutEntities = BeanMapper.mapList(updateInOutVOList, InOutEntity.class);
                    //根据dr分组,因为有的需要删除,有的是修改
                    Map<Integer, List<InOutEntity>> list = inOutEntities.stream().collect(Collectors.groupingBy(InOutEntity::getDr));
                    List<InOutEntity> delList = list.get(1);
                    List<InOutEntity> saveOrUpdList = list.get(0);
                    if (CollectionUtils.isNotEmpty(delList)) {
                        inOutService.removeByIds(delList.stream().map(InOutEntity::getId).collect(Collectors.toList()));
                    }
                    if (CollectionUtils.isNotEmpty(saveOrUpdList)) {
                        inOutService.saveOrUpdateBatch(saveOrUpdList);
                    }

                }
                List<SurplusVO> updateSurplusVOList = useCalculateVO.getUpdateSurplusVOList();
                if (CollectionUtils.isNotEmpty(updateSurplusVOList)) {
                    List<SurplusEntity> surplusEntities = BeanMapper.mapList(updateSurplusVOList, SurplusEntity.class);
                    surplusService.saveOrUpdateBatch(surplusEntities);
                }
                return CommonResponse.success(storeManageVO);
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("操作异常，请刷新重试！");
        } finally {
            if (lock) {
                StoreLockUtil.releaseLock(storeId);
            }
        }
        return CommonResponse.error(res.getMsg());
    }

    /**
     * @param storeManageVO
     * @description: 仓库出库具体实现逻辑逆向实现, 主要调用:出库单删除时
     * @return: com.ejianc.framework.core.response.CommonResponse<com.ejianc.business.store.vo.StoreManageVO>
     * @author songlx
     * @date: 2022/1/25
     */
    @Override
    public CommonResponse<StoreManageVO> handleRollback(StoreManageVO storeManageVO) {
        Long storeId = storeManageVO.getStoreId();
        Boolean lock = StoreLockUtil.getLock(storeId);
        try {
            UseCalculateVO useCalculateVO = surplusService.turnOutRollBackByInOut(storeManageVO, storeManageVO.getOutEffectiveON());
            //3 出库后更新入库\出入关系\库存
            List<FlowVO> updateFlowVOList = useCalculateVO.getUpdateFlowVOList();
            if (CollectionUtils.isNotEmpty(updateFlowVOList)) {
                List<FlowEntity> flowEntities = BeanMapper.mapList(updateFlowVOList, FlowEntity.class);
                flowService.saveOrUpdateBatch(flowEntities);
                // 出库单的处理
                List<Long> sourceIdsForRollBack = storeManageVO.getSourceIdsForRollBack();
                QueryWrapper<FlowEntity> outFlowQueryWrapper = new QueryWrapper<>();
                outFlowQueryWrapper.in("source_id", sourceIdsForRollBack);
                List<FlowEntity> outFlowList = flowService.list(outFlowQueryWrapper);

                if (storeManageVO.getOutEffectiveON()) {
                    outFlowList.forEach(t -> {
                        t.setEffectiveDate(new Date());
                        t.setEffectiveState(StoreCommonConsts.NO);
                    });
                    //出库单生效弃审回滚.则出库单生效状态变成占用态
                    flowService.saveOrUpdateBatch(outFlowList);
                } else {
                    //出库单自由态删除回滚.则删除出库单
                    List<Long> delListIds = outFlowList.stream().map(FlowEntity::getId).collect(Collectors.toList());
                    flowService.removeByIds(delListIds);
                }
            }
            List<InOutVO> updateInOutVOList = useCalculateVO.getUpdateInOutVOList();
            if (CollectionUtils.isNotEmpty(updateInOutVOList)) {
                List<InOutEntity> inOutEntities = BeanMapper.mapList(updateInOutVOList, InOutEntity.class);
                if (storeManageVO.getOutEffectiveON()) {
                    inOutEntities.forEach(t -> {
                        t.setEffectiveDate(new Date());
                        t.setEffectiveState(StoreCommonConsts.NO);
                    });
                    inOutService.saveOrUpdateBatch(inOutEntities);
                } else {
                    List<Long> delListIds = inOutEntities.stream().map(InOutEntity::getId).collect(Collectors.toList());
                    inOutService.removeByIds(delListIds);
                }
            }
            List<SurplusVO> updateSurplusVOList = useCalculateVO.getUpdateSurplusVOList();
            if (CollectionUtils.isNotEmpty(updateSurplusVOList)) {
                List<SurplusEntity> surplusEntities = BeanMapper.mapList(updateSurplusVOList, SurplusEntity.class);
                surplusService.saveOrUpdateBatch(surplusEntities);
            }

            return CommonResponse.success(storeManageVO);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("操作异常，请刷新重试！");
        } finally {
            if (lock) {
                StoreLockUtil.releaseLock(storeId);
            }
        }
    }

}
